//Libraries
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27,20,4);

//Order for buttons and LEDs are Red, Blue, Yellow, Green
int button[4] = {2, 4, 6, 8};
int led[4] = {3, 5, 7, 9};
int tones[4] = {300, 350, 400, 450};

//Arduino related variables (excluding arrays)
int ledTimer = 10;
int buzzer = 11;
int pot = A0;

//Function Prototypes
void extremeMode(int);
int buttonChecker();
bool restart();
void youLose();
void youWin();


void setup()
{
  //LCD initialization
  lcd.init();
  lcd.backlight();

  //Pin mode setting
  for(int i = 0; i<4; i++){
    pinMode(button[i], INPUT);
  }
  for(int i = 0; i<4; i++){
    pinMode(led[i], OUTPUT);
  }
  pinMode(ledTimer, OUTPUT);
  pinMode(buzzer, OUTPUT);
}


void loop()
{
  //Ensuring every run will be randomly generated
  randomSeed(analogRead(A5)); //Analog reading an unused pin to pick up random analog noise

  //Potentiometer and difficulty setup
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Use pot for");
  lcd.setCursor(0, 1);
  lcd.print("difficulties: ");
  for(int i = 5; i>0; i--)
  {
    lcd.setCursor(14, 1);
    lcd.print(i);
    delay(1000);
  }
  int potVal = analogRead(A0);
  potVal = map(potVal, 0, 1014, 0, 20);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Difficulty lvl");
  if(potVal>=19)
  {
    extremeMode(potVal); //The max potVal sometimes reads 19 too, so I include 19 in the ext mode call so someone isn't put on normal mode when having the pot fully at power
    return;
  }
  lcd.setCursor(0, 1);
  lcd.print(potVal);
  delay(2000);

  //Game start countdown
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Game Starting in");
  for(int i = 3; i>0; i--)
  {
    lcd.setCursor(0, 1);
    lcd.print(i);
    delay(1000);
  }

  int maxRounds = 10+potVal;
  int ledNext;
  int pattern[30]; //I read from GPT when needing debugging help that Arduino Unos aren't used to dynamic arrays, so I'll initalize it to the max size and only use what I need

  //Generating the random pattern based on the difficulty chosen
  for(int i = 0; i<maxRounds; i++){
    ledNext = random(0, 4);
    pattern[i] = ledNext;
  }

  for(int round = 1+(potVal/5); round<=maxRounds; round++){
    //Displays how much LEDs will turn on
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("LEDs being used");
    lcd.setCursor(0, 1);
    lcd.print(round);

    //Turns on the LED(s) based on which round and difficulty you are on
    for(int i = 0; i<round; i++){
      digitalWrite(led[pattern[i]], HIGH);
      tone(buzzer, tones[pattern[i]], 500);
      delay(500);
      digitalWrite(led[pattern[i]], LOW);
      delay(500);
    }

    //Player input section
    for(int i = 0; i<round; i++){
      //Timer related variables
      long timer = millis();
      long maxTime = 750*round;

      //Input related variables
      int buttonPressed = 4;
      bool correctButton = false;

      while(millis()-timer <= maxTime){
        buttonPressed = buttonChecker();

        //Led timer related code
        long timePassed = millis()-timer;
        int timerBrightness = map(timePassed, 0, maxTime, 255, 0);
        analogWrite(ledTimer, timerBrightness);
        delay(10);
       
        //True if a button is actually pressed, which then checks if it is the right button in the nested loop
        if(buttonPressed != 4){
          if(buttonPressed == pattern[i]){
            correctButton = true;

            digitalWrite(led[buttonPressed], HIGH);
            tone(buzzer, tones[buttonPressed], 500);
            delay(500);
            digitalWrite(led[buttonPressed], LOW);
            break;
          }
          else{
            youLose();
            return;
          }
        }
      }
      if(correctButton == false){
        analogWrite(ledTimer, 0); //This is to prevent the LED from being really dim but not off when the timer ends
        youLose();
        return;
      }
    }
    delay(500);
  }
  youWin();
}


void extremeMode(int potVal)
{
  potVal = 20; //I don't really want 19 to be used here but since it's a max input for the pot, I gotta account for it
  lcd.setCursor(0, 1);
  lcd.print("Extreme");
  //I asked GPT for a menacing jingle to start this mode with
  int menacingMelody[] = {196, 185, 174, 185, 196, 110, 98};
  int menacingDuration[] = {300, 150, 150, 150, 300, 500, 700};
  for (int i = 0; i < 7; i++) {
    tone(buzzer, menacingMelody[i], menacingDuration[i]);
    delay(menacingDuration[i] + 50);
  }

  //Game start countdown
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Game Starting in");
  for(int i = 3; i>0; i--)
  {
    lcd.setCursor(0, 1);
    lcd.print(i);
    delay(1000);
  }

  int maxRounds = 10+potVal;
  int ledNext;
  int pattern[30];

  //Generating the random pattern based on the difficulty chosen
  for(int i = 0; i<maxRounds; i++){
    ledNext = random(0, 4);
    pattern[i] = ledNext;
  }

  for(int round = 1+(potVal/5); round<=maxRounds; round++){
    //Displays how much LEDs will turn on
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("LEDs being used");
    lcd.setCursor(0, 1);
    lcd.print(round);

    //Turns on the LED(s) based on which round and difficulty you are on
    for(int i = 0; i<round; i++){
      digitalWrite(led[pattern[i]], HIGH);
      tone(buzzer, tones[pattern[i]]+200, 250); //Higher pitch for more emphasis on the difficulty
      delay(250);
      digitalWrite(led[pattern[i]], LOW);
      delay(100);
    }

    //Player inout section
    for(int i = 0; i<round; i++){
      //Timer related variables
      long timer = millis();
      long maxTime = 400*round;

      //Input related variables
      int buttonPressed = 4;
      bool correctButton = false;

      while(millis()-timer <= maxTime){
        buttonPressed = buttonChecker();

        //Led timer related code
        long timePassed = millis()-timer;
        int timerBrightness = map(timePassed, 0, maxTime, 255, 0);
        analogWrite(ledTimer, timerBrightness);
        delay(10);
       
        //True if a button is actually pressed, which then checks if it is the right button in the nested loop
        if(buttonPressed != 4){
          if(buttonPressed == pattern[i]){
            correctButton = true;

            digitalWrite(led[buttonPressed], HIGH);
            tone(buzzer, tones[buttonPressed]+200, 250);
            delay(250);
            digitalWrite(led[buttonPressed], LOW);
            break;
          }
          else{
            youLose();
            return;
          }
        }
      }
      if(correctButton == false){
        analogWrite(ledTimer, 0); //This is to prevent the LED from being really dim but not off when the timer ends
        youLose();
        return;
      }
    }
    delay(500);
  }
  youWin();
}


//This function will return the button number that is pressed, and 4 if nothing is pressed
int buttonChecker()
{
  if(digitalRead(button[0]) == HIGH){
    return 0;
  }
  else if(digitalRead(button[1]) == HIGH){
    return 1;
  }
  else if(digitalRead(button[2]) == HIGH){
    return 2;
  }
  else if(digitalRead(button[3]) == HIGH){
    return 3;
  }
  else{
    return 4;
  }
}


//This function is used so the player can restart whenever they want and aren't forced on a timer after winning or losing
bool restart()
{
  if(digitalRead(button[0]) == HIGH){
    return true;
  }
  else if(digitalRead(button[1]) == HIGH){
    return true;
  }
  else if(digitalRead(button[2]) == HIGH){
    return true;
  }
  else if(digitalRead(button[3]) == HIGH){
    return true;
  }
  else{
    return false;
  }
}


void youLose()
{
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Game over :(");

  tone(buzzer, 400, 200);
  delay(250);
  tone(buzzer, 300, 200);
  delay(250);
  tone(buzzer, 200, 400);
  delay(500);

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Press any button");
  lcd.setCursor(0,1);
  lcd.print("to try again");

  while(restart()==false){
    for(int i = 0; i<4; i++)
    {
      digitalWrite(led[i], HIGH);
    }
    delay(200);
    for(int i = 0; i<4; i++)
    {
      digitalWrite(led[i], LOW);
    }
    delay(200);
  }
}


void youWin()
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("You win! :D");

  //The following buzzer code was my asking ChatGPT for a nice win jingle to play for when the game is won
  int melody[] = {523, 659, 784, 1047};
  int noteDuration[] = {200, 200, 200, 400};

  for(int i = 0; i<4; i++){
    tone(buzzer, melody[i], noteDuration[i]);
    delay(noteDuration[i] + 50);
  }

  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Press any button");
  lcd.setCursor(0,1);
  lcd.print("to play again");

  while(restart() == false){
    for(int i = 0; i<4; i++){
      restart(); //Restart is called here to prevent a possibly half second long delay of waiting for the for loop to end
      digitalWrite(led[i], HIGH);
      delay(50);
      digitalWrite(led[i], LOW);
      delay(50);
    }
  }
}